package inclesra.projecte.local;

import java.io.IOException;
import java.util.Date;
import java.util.List;

import android.app.Activity;
import android.content.Intent;
import android.location.Address;
import android.location.Criteria;
import android.location.Geocoder;
import android.location.Location;
import android.location.LocationListener;
import android.location.LocationManager;
import android.os.Bundle;
import android.util.Log;
import android.view.Menu;
import android.view.MenuInflater;
import android.view.MenuItem;
import android.widget.ProgressBar;
import android.widget.TextView;
import android.widget.Toast;

public class LocalitzacioActivity extends Activity implements LocationListener{
	private LocationManager elmeuManager;
	private TextView tv;
	private static final String TAG = "LocalitzacioGPS";
	private Geocoder geocoder;
    private boolean gps_habilitat=false;
	private boolean xarxa_habilitat=false;
	private ProgressBar progressBar;
	
	@Override
	public void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		setContentView(R.layout.locagps);

		// Visualitzem barra de progres
		 progressBar = (ProgressBar) findViewById(R.id.progressBar1);
         progressBar.setVisibility(ProgressBar.VISIBLE);
         
		// Apuntem a un TextView per escriure informacio més endevant
		tv = (TextView) findViewById(R.id.textView1);
		tv.setText("Buscant ..");
		
		// Missatge en consola de logs
		Log.v(TAG, "Abans getSystemService");
		
		// Configuracio del LocationManager
		elmeuManager = (LocationManager) getSystemService(LOCATION_SERVICE);
		
		gps_habilitat = false;
		xarxa_habilitat = false;
		
		// Consultem si esta disponible el GPS o la Xarxa
		try {gps_habilitat=elmeuManager.isProviderEnabled(LocationManager.GPS_PROVIDER);} 
		catch(Exception ex){}
		try {xarxa_habilitat=elmeuManager.isProviderEnabled(LocationManager.NETWORK_PROVIDER);} 
		catch(Exception ex){}
						
				
		// Utilitzat per obtenir informacio de l'adreça fisica de la posicio geografica
		geocoder = new Geocoder(this);
		
		// Despres de Oncreate automaticament s'executa el metode onResume
	}

	@Override
	protected void onDestroy() {
		super.onDestroy();
		elmeuManager.removeUpdates(this);
	}
	
	@Override
	protected void onPause() {
		super.onPause();
		elmeuManager.removeUpdates(this);
	}
	
	@Override
	protected void onResume() {
		Log.v(TAG, "Entrant onResume");
		startListening();
		super.onResume();
	}

	@Override
	protected void onStop() {
		super.onStop();
		elmeuManager.removeUpdates(this);
	}
	
	// funcions de start stop segons els canvis 
	private void startListening() {
		
		Location net_loc=null, gps_loc=null;
						
		if (gps_habilitat)
    	{
    		elmeuManager.requestLocationUpdates(LocationManager.GPS_PROVIDER, 0, 0, this);
    		Toast.makeText(this, "Detectant ubicació GPS", Toast.LENGTH_SHORT).show();
    		//** Deshabilitat, no mostrara la ultima posició coneguda 
			//** per no donar lloca a confusions
			//gps_loc = elmeuManager.getLastKnownLocation(LocationManager.GPS_PROVIDER);
    		
    	}
		else
			if (xarxa_habilitat)
			{
				elmeuManager.requestLocationUpdates(LocationManager.NETWORK_PROVIDER, 0, 0, this);
				Toast.makeText(this, "Detectant ubicació Wifi", Toast.LENGTH_SHORT).show();
				//** Deshabilitat, no mostrara la ultima posició coneguda 
				//** per no donar lloca a confusions
				//net_loc = elmeuManager.getLastKnownLocation(LocationManager.NETWORK_PROVIDER);
				
			}
			else
			{
				Toast.makeText(this, "No s'ha detectat controlador", Toast.LENGTH_SHORT).show();
			}
					
					
	}

	private void Wifi()
	{
		//elmeuManager.getProvider(elmeuManager.getBestProvider(createCoarseCriteria(), xarxa_habilitat));
		xarxa_habilitat=elmeuManager.isProviderEnabled(LocationManager.NETWORK_PROVIDER);
		if (xarxa_habilitat) 
		Toast.makeText(this, "Localització Xarxa habilitada", Toast.LENGTH_SHORT).show();
		else
			Toast.makeText(this, "Localització Xarxa deshabilitada", Toast.LENGTH_SHORT).show();
		gps_habilitat = false;
		startListening();
	}
			
	
	private void GPS()
	{
		//elmeuManager.getProvider(elmeuManager.getBestProvider(createCoarseCriteria(), xarxa_habilitat));
		gps_habilitat=elmeuManager.isProviderEnabled(LocationManager.GPS_PROVIDER);
		if (gps_habilitat) 
		Toast.makeText(this, "Localització GPS habilitada", Toast.LENGTH_SHORT).show();
		else
			Toast.makeText(this, "Localització GPS deshabilitada", Toast.LENGTH_SHORT).show();
		xarxa_habilitat = false;
		startListening();
	}
			
	
	
	@Override
	public void onLocationChanged(Location location) {
		// Quan cambia la posicio la mostrem la informacio en un textview
		if (location != null) {
			mostra_informacio(location);
		}
	}

	@Override
	public void onProviderDisabled(String provider) {}

	@Override
	public void onProviderEnabled(String provider) {}

	@Override
	public void onStatusChanged(String provider, int status, Bundle extras) {}
	
	

	// Procediments tecla menu
	
	@Override
	public boolean onCreateOptionsMenu(Menu menu) {
	// TODO Auto-generated method stub
	
	super.onCreateOptionsMenu(menu);
	MenuInflater boto_menu_android = getMenuInflater();
	boto_menu_android.inflate(R.menu.menuinflater_localitzacio, menu);
		return true;
	}
	public boolean onOptionsItemSelected(MenuItem item){
		switch (item.getItemId()) {
		case R.id.menuTornar:
			finish();
			startActivity(new Intent("inclesra.projecte.local.ELMEUMENU"));
			return true;
		case R.id.Wifi:
			Wifi();
			return true;
		case R.id.GPS:
			GPS();
			return true;
		}
			
		return false;

	}
	private void mostra_informacio(Location location)
	{
		// Al iniciar mostrem la ultima posicio coneguda
				String s = "";
												
				if (location == null)
					s += " Localització desconeguda ";
				else
				{
					s += "Localització: "+"\n";
					s += "\n";
					s += "Proveidor: "+ location.getProvider()  +"\n";
					s += "Time: "        + new Date(location.getTime()).toString() + "\n";
					s += "\tLatitude:  " + location.getLatitude()  + "\n";
					s += "\tLongitude: " + location.getLongitude() + "\n";
					s += "\tAltitud: " + location.getAltitude() + "\n";
					s += "\tBearing:  " + location.getBearing()  + "\n";
					s += "\tAccuracy:  " + location.getAccuracy()  + "\n";
					
					
					// Obte l'adreça amb el geocoder
					
					try {
					
					if (gps_habilitat)
					{
					
						s += "\tUbicació per GPS " + "\n";
					// *** El geocoder sembla funcionar només per wifi 
					//	*** amb GPS sol no va, potser GPS+GPRS amb dades pot funcionar  
					//	List<Address> addresses = geocoder.getFromLocation(location.getLatitude(), location.getLongitude(), 1);
					//	if (addresses.size() == 1)
					//	{
					//			s += "\tDirecció:  " + addresses.get(0)  + "\n";
					//	}
													
					}
					else
					{
						// Es wifi
						List<Address> addresses = geocoder.getFromLocation(location.getLatitude(), location.getLongitude(), 10);
						if (addresses.size() >0)
						{
							for (Address address : addresses) {
								s += "\tDirecció:  " + address.getAddressLine(0)  + "\n";
								}
						}
						}
					
					}
					
					catch
					(IOException error) {
						Log.e("onLocationChanged", "No es pot obtenir dades direccio", error);
						s += " No es pot obtenir dades direccio ";
					}
					
					progressBar.setVisibility(ProgressBar.GONE);
													
				}
				tv.setText(s);
	}	
}